// UrsADC
//
// Autor: http://UllisRoboterSeite.de
// Doku:  http://bienonline.magix.net/public/esp8266-adc.html
// Created: 2017-01-08

#include <UrsAdc.h>
#ifdef __cplusplus
extern "C" {
#endif

#include <user_interface.h>

#ifdef __cplusplus
} // extern "C"
#endif

// Initiales read(). Die erste Wandlung dauert lnger (ca. 900s). 
// Wahrscheinlich finden hier interne Initialisierungen statt.
// Liefert ebenfalls den ersten Basiswert fr das Hysterese-Fenster.
UrsAdcClass::UrsAdcClass()
{ read(); 
}

// Mittelt ADC-Werte zur besseren Stabilitt der ADC-Reading.
// Es werden Samples aufeinanderfolgende Messwerte aufgenommen
// Der jeweils hchste und niedrigste wird verworfen.
// Der Messwert ist der Mittelwert aus den Samples-2 verbliebenen Werten.
// Der Messwert wird anschlieend gegen die eingestellte Hysterese geprft.
uint16_t UrsAdcClass::read()
{ // 6 ADC-Werte einlesen, in temp[] zwischenspeichern, Min und Max ermitteln
  for (uint8_t i = 0; i < Samples; i++)
  { uint16_t v = system_adc_read();
    TempValues[i] = v;
    if (v > MaxVal) // Maximum merken
    { MaxVal = v;   // Wert
      MaxInd = i;   // Index
    }
    if (v < MinVal) // Minimum merken
    { MinVal = v;   // Wert
      MinInd = i;   // Index
    }
  }

  if (MinInd == MaxInd) // Alle Werte identisch
    LastValue = MinVal;
  else
  { uint16_t sum = 0;

    for (uint8_t i = 0; i < Samples; i++)
    { if (i == MaxInd) continue;
      if (i == MinInd) continue;
      sum += TempValues[i];
    } // for
    LastValue = sum / (Samples-2);
  } // else

  uint16_t top = LastCommitedValue + Hysteresis; // obere Schranke

  if (SpecialZeroTreatment) // Sonderbehandlung fr 0.
    if (LastCommitedValue == 0)
      top += Hysteresis / 2; // 1,5-facher Wert fr die Hysterese

  uint16_t bot = 0;          //untere Schranke.
  if (LastCommitedValue > Hysteresis) // Wegen uint16_t bei der Subtraktion aufpassen!
    bot = LastCommitedValue - Hysteresis; // sonst 0.

  if ((int32_t)LastValue > top || (int32_t)LastValue < bot)
    LastCommitedValue = LastValue;

  if (SpecialZeroTreatment) // Sonderbehandlung fr 0.
    if (LastCommitedValue < Hysteresis) 
      LastCommitedValue = 0;

  return LastCommitedValue;
}

UrsAdcClass UrsAdc;

